# -*- coding: utf-8 -*-

import unittest

from babelvueextractor.extract import _get_messages
from babelvueextractor.utils import is_protected_type, force_text, smart_split
import datetime


class TestUtils(unittest.TestCase):
    def test_get_messages(self):
        self.assertEqual(
            _get_messages('"foo, bar", "bar"'),
            [u'foo, bar', u'bar']
        )
        self.assertRaises(
            ValueError,
            _get_messages,
            "a + b"
        )

    def test_is_protected_type_numeric(self):
        assert is_protected_type(5)
        assert is_protected_type(5L)
        assert is_protected_type(5.0)

    def test_is_protected_type_date(self):
        assert is_protected_type(datetime.datetime.now())
        assert is_protected_type(datetime.date.today())
        assert is_protected_type(datetime.datetime.now().time())

    def test_is_protected_type_sequences(self):
        assert not is_protected_type(tuple())
        assert not is_protected_type(dict())
        assert not is_protected_type(set())

    def test_is_protected_type_func(self):
        assert not is_protected_type(lambda x: x)

    def test_force_text_list(self):
        self.assertEqual(force_text(['a', 'b']), u"['a', 'b']")

    def test_force_text_datetime(self):
        self.assertEqual(u'2015-11-21 11:05:05',
                         force_text(datetime.datetime(year=2015, day=21, month=11, hour=11, minute=5, second=5)))

    def test_force_text_datetime_strings_only(self):
        t = datetime.datetime(2015, 11, 21, 11, 5, 5)
        self.assertEqual(
            t,
            force_text(t, strings_only=True))

    def test_force_text_str(self):
        self.assertEqual(force_text('hello'), u"hello")

    def test_force_text_unicode(self):
        self.assertEqual(force_text(u'привет'), u'привет')

    def test_smart_split(self):
        self.assertEqual(
            list(smart_split(r'This is "a person\'s" test.')),
            ['This', 'is', '"a person\\\'s"', 'test.'])

        self.assertEqual(
            list(smart_split(r"Another 'person\'s' test.")),
            ['Another', "'person\\'s'", 'test.'])

        self.assertEqual(
            list(smart_split(r'A "\"funky\" style" test.')),
            ['A', '"\\"funky\\" style"', 'test.'])



import unittest

from babelvueextractor.extract import func_re


class TestFuncRegExp(unittest.TestCase):
    def test_gettext(self):
        content = 'gettext("Hello")'
        self.assertDictEqual(func_re.match(content).groupdict(), {
            'funcname': 'gettext',
            'messages': '"Hello"'
        })

    def test_undersoce(self):
        content = '_("Hello")'
        self.assertDictEqual(func_re.match(content).groupdict(), {
            'funcname': '_',
            'messages': '"Hello"'
        })

    def test_funct_without_on_brace(self):
        content = ' gettext("Hello"'
        assert func_re.match(content) is None




# -*- coding: utf-8 -*-
import unittest

from babel.messages.extract import DEFAULT_KEYWORDS, extract

from babelvueextractor.extract import extract_vue
from babelvueextractor.tests.utils import FileMock


class TestMessagesExtractor(unittest.TestCase):
    def test_empty_template(self):
        template = FileMock("")
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(), list(result))

    def test_no_messages(self):
        template = FileMock("""
        <div>
            <h1>Foo</h1>
            <p>Lore ipsum delore ...</p>
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(), list(result))

    def test_gettext(self):
        template = FileMock("""
        <div>
            {{ gettext('Foo') }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(result), [
            (3, u'gettext', u"Foo", [])
        ])

    def test_underscore(self):
        template = FileMock("""
        <div>
            {{ _("Bar") }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(result), [
            (3, '_', u'Bar', [])
        ])

    def test_token_without_content(self):
        template = FileMock("""
            {{  }}
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(result), [])

    def test_commas(self):
        template = FileMock("""
            {{ gettext('Hello, User') }}
            {{ gettext("You're") }}
        """)

        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(result), [
            (2, 'gettext', u'Hello, User', []),
            (3, 'gettext', u"You're", [])
        ])

    def test_babel(self):
        method = 'babelvueextractor.extract.extract_vue'
        fileobj = open('babelvueextractor/tests/templates/for_babel.vhtml')
        result = extract(method, fileobj)

        self.assertListEqual(list(result), [
            (1, u'Привет, User', [], None),
            (2, u'привет123', [], None)
        ])

    def test_gettext_with_parameter(self):
        template = FileMock("""
        <li class="select__option"
            v-for="season in rankSeasons"
            @click="toggleSeasonAction(season)">
            {{ gettext('{number} season').replace("{number}", season) }}
        </li>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertEqual(list(result), [
            (5, u'gettext', u'{number} season', [])
        ])

    def test_directives(self):
        template = FileMock("""
        <div v-text="gettext('Sometext')"></div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertListEqual(list(result), [(2, u'gettext', u'Sometext', [])])

    def test_directives_with_inner_tag(self):
        template = FileMock("""
        <div v-text="gettext('Sometext')">
        {{ gettext('Hello') }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], {})
        self.assertListEqual(list(result), [
            (2, u'gettext', u'Sometext', []),
            (3, u'gettext', u'Hello', [])
        ])



import re
import six


def is_protected_type(obj):
    """Determine if the object instance is of a protected type.

    Objects of protected types are preserved as-is when passed to
    force_text(strings_only=True).
    """
    import datetime
    return isinstance(
        obj,
        six.integer_types + (type(None), float, datetime.datetime, datetime.date, datetime.time)
    )


def force_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Similar to smart_text, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.

    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first, saves 30-40% when s is an instance of
    # six.text_type. This function gets called often in that setting.
    if isinstance(s, six.text_type):
        return s
    if strings_only and is_protected_type(s):
        return s
    try:
        if not isinstance(s, six.string_types):
            if hasattr(s, '__unicode__'):
                s = s.__unicode__()
            else:
                if six.PY3:
                    if isinstance(s, bytes):
                        s = six.text_type(s, encoding, errors)
                    else:
                        s = six.text_type(s)
                else:
                    s = six.text_type(bytes(s), encoding, errors)
        else:
            # Note: We use .decode() here, instead of six.text_type(s, encoding,
            # errors), so that if s is a SafeBytes, it ends up being a
            # SafeText at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise UnicodeDecodeError(s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join([force_text(arg, encoding, strings_only,
                                     errors) for arg in s])
    return s


smart_split_re = re.compile(r"""
    ((?:
        [^\s'"]*
        (?:
            (?:"(?:[^"\\]|\\.)*" | '(?:[^'\\]|\\.)*')
            [^\s'"]*
        )+
    ) | \S+)
""", re.VERBOSE)


def smart_split(text):
    r"""
    Generator that splits a string by spaces, leaving quoted phrases together.
    Supports both single and double quotes, and supports escaping quotes with
    backslashes. In the output, strings will keep their initial and trailing
    quote marks and escaped quotes will remain escaped (the results can then
    be further processed with unescape_string_literal()).

    >>> list(smart_split(r'This is "a person\'s" test.'))
    ['This', 'is', '"a person\\\'s"', 'test.']
    >>> list(smart_split(r"Another 'person\'s' test."))
    ['Another', "'person\\'s'", 'test.']
    >>> list(smart_split(r'A "\"funky\" style" test.'))
    ['A', '"\\"funky\\" style"', 'test.']
    """
    text = force_text(text)
    for bit in smart_split_re.finditer(text):
        yield bit.group(0)



import re
from ast import literal_eval

from babelvueextractor.lexer import (
    Lexer,
    TOKEN_VAR,
    TOKEN_CONST,
    TOKEN_COMMENT,
    TOKEN_RAW_HTML,
    TOKEN_DOUBLE_WAY_BINDING,
    TOKEN_DIRECTIVE
)
from babelvueextractor.utils import force_text

TOKENS = [TOKEN_VAR, TOKEN_CONST, TOKEN_COMMENT, TOKEN_RAW_HTML, TOKEN_DOUBLE_WAY_BINDING, TOKEN_DIRECTIVE]

func_re = re.compile('(?P<funcname>(\w+|_))\((?P<messages>.*?)\)+?')


def _get_messages(raw_messages_string):
    try:
        messages = literal_eval(u'[%s]' % unicode(raw_messages_string))
    except:
        raise
    messages = map(force_text, messages)

    if len(messages) == 1:
        return messages[0]

    return messages


def extract_vue(fileobj, keywords, comment_tags, options):
    """Extract messages from Vue template files.

    :param fileobj: the file-like the messages should be extracted from
    :param keywords: a list of keywords (i.e. function names) that should be recognize as translation functions
    :param comment_tags: a list of translator tags to search for and include in the results
    :param options: a dictionary of additional options (optional)
    :return: an iterator over ``(lineno, funcname, message, comments)``
    :rtype: ``iterator``
    """

    intrans = False
    inplural = False
    incomment = False
    singular = []
    plural = []
    lineno = 1
    template = None

    for t in Lexer(unicode(fileobj.read(), encoding=options.get('encoding', 'utf-8')), None).tokenize():
        if t.token_type in TOKENS:
            try:
                matched = func_re.match(t.contents).groupdict()
            except AttributeError:
                pass
            else:
                messages = _get_messages(matched['messages'])
                # messages = matched['messages']

                yield t.lineno, matched['funcname'], messages, []
